function [bus, gen, branch, f, success, et] = opf(baseMVA, bus, gen, gencost, ...
					branch, Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%OPF  Solves an optimal power flow.
%   [bus, gen, branch, f, success, et] = opf(baseMVA, bus, gen, gencost, ...
%                   branch, Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%   Assumes that Ybus, Yf, Yt, V, ref, pv, pq are consistent with
%   (or override) data in bus, gen, branch.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialization -----
%% constants
npts = 6;				%% number of points to evaluate when converting
						%% polynomials to piece-wise linear

%% initialize arguments
if nargin < 15
	verbose = 0;		%% default: don't print stuff
	if nargin < 14
		alg = 0;		%% use default algorithm for this type of cost model
	end
end

%% define named indices into data matrices
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;

%% get cost model, check consistency
model = gencost(:, MODEL);
i_pwln = find(model == PW_LINEAR);
i_poly = find(model == POLYNOMIAL);
if any(i_pwln) & any(i_poly) & verbose
	fprintf('not all generators use same cost model, all will be converted to piece-wise linear\n');
end

%% set default algorithm if necessary
if alg == 0
	if find(model ~= PW_LINEAR & model ~= POLYNOMIAL)
		error('unknown generator cost model');
	elseif any(i_pwln)		%% some piece-wise linear, use appropriate alg
		alg = 201;
	else					%% must all be polynomial
		alg = 101;
	end
end

%% check cost model/algorithm consistency
if any( i_pwln ) & alg < 200
	error(sprintf('algorithm %d does not handle piece-wise linear cost functions', alg));
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%%	Eventually put code here to fit polynomials to piece-wise linear as needed.
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
if any(i_poly)  & alg >= 200
	%% convert polynomials to piece-wise linear by evaluating at zero and then
	%% at evenly spaced points between Pmin and Pmax
	if verbose
		fprintf('converting from polynomial to piece-wise linear cost model\n');
	end
	oldgencost = gencost;
	[m, n] = size(gencost(i_poly, :));								%% size of piece being changed
	gencost(i_poly, MODEL) = PW_LINEAR * ones(m, 1);				%% change cost model
	gencost(i_poly, COST:n) = zeros(size(gencost(i_poly, COST:n))); %% zero out old data
	gencost(i_poly, N) = npts * ones(m, 1);							%% change number of data points
	
	for i = 1:m
		ig = i_poly(i);		%% index to gen
		Pmin = gen(ig, PMIN);
		Pmax = gen(ig, PMAX);
		if gen(ig, PMIN) == 0
			step = (Pmax - Pmin) / (npts - 1);
			xx = [Pmin:step:Pmax];
		else
			step = (Pmax - Pmin) / (npts - 2);
			xx = [0 Pmin:step:Pmax];
		end
		yy = totcost(oldgencost(ig, :), xx);
		gencost(ig, 	COST:2:(COST + 2*(npts-1)	 )) = xx;
		gencost(ig, (COST+1):2:(COST + 2*(npts-1) + 1)) = yy;
	end
end

%%----- run opf -----
%% start timer
t0 = clock;

%%	Dispatch to appropriate OPF algorithm
if usesOT(alg)					%% based on Optimization Toolbox
	[V, Pg, f, lamda, success] = OTopf(baseMVA, bus, gen, gencost, branch, ...
					Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose);
elseif usesLP(alg)				%% LP-based
	[V, Pg, f, lamda, success] = LPopf(baseMVA, bus, gen, gencost, branch, ...
					Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose);
else
	error('unknown algorithm');
end

%% update bus, gen, branch with solution info
[bus, gen, branch] = opfsoln(baseMVA, bus, gen, branch, ...
						Ybus, Yf, Yt, V, Pg, lamda, ref, pv, pq, alg);

%% compute elapsed time
et = etime(clock, t0);

return;
