function runopf(casename, alg, verbose, pretty, ugly)
%RUNOPF  Runs an optimal power flow.
%
%   runopf(casename, alg, verbose, pretty, ugly)
%   
%   Runs an optimal power flow where casename is the name of the m-file
%   (without the .m extension) containing the opf data, and alg specifies
%   the algorithm to use. If verbose is true it prints some extra things
%   while it's solving, and if pretty is true it pretty prints the output.
%   If ugly is true it prints raw data for our Perl database interface.
%   Default parameters are 'case', 0, 1, 1 and 0.
%   The alg parameter is interpreted as follows:
%       100 - Opt Tbx standard formulation, poly cost, fixed Vg
%       101 - Opt Tbx standard formulation, poly cost, var Vg
%       102 - Opt Tbx QCCV formulation, poly cost, fixed Vg
%       103 - Opt Tbx QCCV formulation, poly cost, var Vg
%       200 - Opt Tbx PLCCV formulation, p/w linear cost, fixed Vg
%       201 - Opt Tbx PLCCV formulation, p/w linear cost, var Vg
%       220 - Dense LP formulation, p/w linear cost, var Vg (lp.m solver)
%       221 - Dense LP formulation, p/w linear cost, var Vg (bpmpd solver)
%       222 - Dense LP formulation, p/w linear cost, var Vg (lp_solve solver)
%       223 - Dense LP formulation, p/w linear cost, var Vg (loqo solver)
%       224 - Dense LP formulation, p/w linear cost, var Vg (sol_qps solver)
%       240 - Sparse LP formulation, p/w linear cost, var Vg (lp.m solver)
%       241 - Sparse LP formulation, p/w linear cost, var Vg (bpmpd solver)
%       242 - Sparse LP formulation, p/w linear cost, var Vg (lp_solve solver)
%       243 - Sparse LP formulation, p/w linear cost, var Vg (loqo solver)
%       244 - Sparse LP formulation, p/w linear cost, var Vg (sol_qps solver)
%   See the README for more info on the algorithms.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% define named indices into data matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;

%% default arguments
if nargin < 5
	ugly = 0;							%% don't print the raw stuff
	if nargin < 4
		pretty = 1;						%% print the pretty stuff
		if nargin < 3
			verbose = 1;				%% be verbose by default
			if nargin < 2
				alg = 0;				%% use default algorithm
				if nargin < 1
					casename = 'case';	%% default data file is 'case.m'
				end
			end
		end
	end
end

%% read data & convert to internal bus numbering
[baseMVA, bus, gen, branch, area, gencost] = feval(casename);
[i2e, bus, gen, branch, area] = ext2int(bus, gen, branch, area);

%% get bus index lists of each type of bus
[ref, pv, pq] = bustypes(bus, gen);

%% build admittance matrices
[Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);

%% generator info
on = find(gen(:, GEN_STATUS));				%% which generators are on?

%% initialize V and Pg from data from case file
V0	= bus(:, VM) .* exp(sqrt(-1) * pi/180 * bus(:, VA));
V0(gen(on, GEN_BUS)) = gen(on, VG) ./ abs(V0(gen(on, GEN_BUS))).* V0(gen(on, GEN_BUS));
Pg0	= gen(on, PG) / baseMVA;

%% run the optimal power flow
[bus, gen, branch, f, success, et] = opf(baseMVA, bus, gen, gencost, branch, ...
					Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose);

%% convert back to original bus numbering & print results
[bus, gen, branch, area] = int2ext(i2e, bus, gen, branch, area);
printopf(bus, gen, branch, f, success, et, 1, pretty, ugly);

return;
